package ru.karelia.cs.petrsugen.application;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.*;
import java.util.logging.Level;
import ru.karelia.cs.petrsugen.util.Logger;
import ru.karelia.cs.petrsugen.application.Generator;

/**
 * 
 * 
 */
public class Main {

    Logger log;
    private static final String BLOCK_FORMAT = "--- %s ---";
    private static final String BLOCK_NAME_CLASSES = "Classes";
    private static final String BLOCK_NAME_DATA_PROPERTIES = "Data properties";
    private static final String BLOCK_NAME_OBJECT_PROPERTIES = "Object properties";

    private enum EntityType {

        CLASSES, DATA_PROPERTIES, OBJECT_PROPERTIES, NOT_SPECIFIED
    };

    /**
     * @param args
     *            the command line arguments
     */
    public static void main(String[] args) {
        //Analyze input parameters

        boolean gui = false;

        List<String> inputFiles = new LinkedList<String>();
        List<String> filterFiles = new LinkedList<String>();

        Set<String> filterIRI = null;

        String outputName = null;
        String outputPath = null;
        String mainFile = "";
        String makeFile = "";
        String doxyFile = "";
        String testProgram = null;
        String triplestore = null;
        String handlerType = "C";

        if (args.length == 0) {
            //gui is disabled
            //printHelp();
            // GUI disabled !
            //Gui g = new Gui();
            //g.setVisible(true);
            //gui = true;
        } else if (args.length == 1) {
            if (args[0].equals("-h") || args[0].equals("--help") || args[0].equals("/?")) { // Now we have it all
                printHelp();
            } else {
                inputFiles.add(args[0]);
            }
        } else if (args.length >= 1) {
            //inputFile = args[0];
            int current = 0; // parameter to be evaluated
            while (args.length > current) {
                if (args[current].equals("-o")) {
                    current++;
                    outputPath = new File(args[current]).getAbsolutePath();
                    outputPath += File.separator;
                } else if (args[current].equals("-t")) {
                    current++;
                    triplestore = args[current];
                } else if (args[current].equals("-tp")) {
                    current++;
                    testProgram = args[current];

                } else if (args[current].equals("-m")) {
                    current++;
                    mainFile = args[current];
                } else if (args[current].equals("-k")) {
                    current++;
                    makeFile = args[current];

                } else if (args[current].equals("-d")) {
                    current++;
                    doxyFile = args[current];
                } else if (args[current].equals("-h")) {
                    current++;
                    handlerType = args[current];
                } else if (args[current].equals("-n")) {
                    current++;
                    outputName = args[current];
                } else if (args[current].equals("-f")) {
                    current++;
                    filterFiles.add(args[current]);
                } else {
                    inputFiles.add(args[current]);
                    //    printHelp();
                }
                current++;
            }
        }


        if ((outputPath == null) || (inputFiles.isEmpty() == true)) {
            printHelp();
        } else {

            boolean isFilesExists = true;

            for (String file : inputFiles) {
                File f = new File(file);
                System.out.println("Input file: " + file);

                if (f.exists() == false) {
                    isFilesExists = false;
                    break;
                }
            }


            File p = new File(outputPath);
            if (isFilesExists == true && (p.exists() == true)) {

                if (filterFiles.isEmpty() != true) {
                    filterIRI = new HashSet<String>();
                }

                for (String path : filterFiles) {
                    parseFilterFile(path, filterIRI);
                }
                

                System.out.println("Output path: " + outputPath);
                Generator generator = new Generator();

                generator.runGenerator(new Logger("Code generator", true),
                        inputFiles, outputPath, triplestore, testProgram, mainFile, makeFile, doxyFile, handlerType, outputName, filterIRI);
            } else {
                if (isFilesExists == false) {
                    System.out.println("Some input files are not exists: ");
                }
                if (!p.exists() == false) {
                    System.out.println("outputPath does not exists: " + outputPath);
                }


            }
        }
        //}
    }

    private static void parseFilterFile(String file, Set<String> filterData) {

        java.util.logging.Logger logger = java.util.logging.Logger.getLogger(Main.class.getName());
        BufferedReader reader;
        try {
            reader = new BufferedReader(new FileReader(file));
        } catch (FileNotFoundException ex) {
            logger.log(Level.SEVERE, "Can't open file: " + file, ex);
            return;
        }

        String classBlock = String.format(BLOCK_FORMAT, BLOCK_NAME_CLASSES);
        String dProertyBlock = String.format(BLOCK_FORMAT, BLOCK_NAME_DATA_PROPERTIES);
        String oProertyBlock = String.format(BLOCK_FORMAT, BLOCK_NAME_OBJECT_PROPERTIES);


        EntityType type = EntityType.NOT_SPECIFIED;

        String line;

        try {

            while ((line = reader.readLine()) != null) {
                if (line.startsWith(classBlock)) {
                    type = EntityType.CLASSES;
                } else if (line.startsWith(dProertyBlock)) {
                    type = EntityType.DATA_PROPERTIES;
                } else if (line.startsWith(oProertyBlock)) {
                    type = EntityType.OBJECT_PROPERTIES;
                } else if (type != EntityType.NOT_SPECIFIED) {
                    logger.log(Level.INFO, "Entity: {0} ({1})", new Object[]{line, type});
                    filterData.add(line);
                }
            }
            reader.close();
        } catch (IOException ex) {
            java.util.logging.Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private static void printHelp() {
        // Now we have it all
        System.out.println("SmartSlog Code generator version 0.4alpha.");
        System.out.println("Usage: java -jar CodeGenerator file1 [file2 ... fileN] [-o path] [options]");
        System.out.println("");
        System.out.println("Options:");
        //System.out.println("    -f file          Input ontology in XML/RDF format. You can use this option many times.");
        System.out.println("    -n name          Output file name. Defaults name of firts imput file.");
        System.out.println("    -o path          Output path. Defaults to path of input file.");
        //System.out.println("    -t triplestore 	 enabled(default) / disabled.");
        //System.out.println("    -tp TestProgram	 enabled / disabled (default).");
        System.out.println("    -m main          Generate KP template(kp), enabled(default) disabled.");
        System.out.println("    -k Makefile      Generate Makefile, enabled(default) / disabled.");
        System.out.println("    -h handler       Handler type(C, C# or C# with static class ) for generate: C (default) / CS / CSstatic.");
        System.out.println("    -f fiter         Filter file contains IRI for entities that will be used in the generating prociss.");
        //System.out.println("    -d Doxyfile 	 Generate doxyfile, enabled(default) / disabled.");
    }
}
