/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package ru.karelia.cs.petrsugen.handlers.cHandlers;

import java.util.LinkedList;
import java.util.Map;

import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.ontology.ObjectProperty;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

import ru.karelia.cs.petrsugen.application.AppConfiguration;
import ru.karelia.cs.petrsugen.framework.CCodeModelContainer;
import ru.karelia.cs.petrsugen.templates.TemplateResourceFile;
import ru.karelia.cs.petrsugen.util.TemplateString;


/**
 *
 *
 */

public class CModelHandler {
	static String headerHeaderTemplateName = TemplateResourceFile.getTemplate("CModuleDefinition");
	static String implementationTemplateName = TemplateResourceFile.getTemplate("CModuleImplementation");    
    static String COntologyTriple = TemplateResourceFile.getTemplate("COntologyTriple");    
    static String CObjectPropertyEnumDeclaration = TemplateResourceFile.getTemplate("CObjectPropertyEnumDeclaration");
    
    

    public static LinkedList<LinkedList<String>> getTripleOntology(OntModel model) {
    	LinkedList<LinkedList<String>> triples = new LinkedList<LinkedList<String>>();
    	if (model != null) {
            /*
            this.classes = new LinkedList<ClassTree>();
            this.deserializeModel(model);
             */
            StmtIterator iterator = model.listStatements();
            LinkedList<String> tripleList;
            Triple triple = null;
            int cntStatements = 0;
            int cntStatementsInBaseModel = 0;
            while (iterator.hasNext()) {
                tripleList = new LinkedList<String>();
                Statement statement = iterator.nextStatement();
                cntStatements++;
                if ( model.isInBaseModel(statement) ) {
                    //System.out.println("Yes "+cntStatementsInBaseModel);
                    cntStatementsInBaseModel++;
                    triple = statement.asTriple();

                    tripleList.add(triple.getSubject().toString());
                    tripleList.add(triple.getPredicate().toString());
                    tripleList.add(triple.getObject().toString());
                    if (triple.getObject().isLiteral()) {
                        tripleList.add("True");
                    } else if (triple.getObject().isURI()) {
                        tripleList.add("False");
                    } else if (triple.getObject().isConcrete()) {
                        tripleList.add("False");
                    } else if (triple.getObject().isVariable()) {
                        tripleList.add("False");
                    } else if (triple.getObject().isBlank()) {
                        tripleList.add("False");
                    }
                    triples.add(tripleList);
                } else {
                    //System.out.println("No "+ (cntStatements-cntStatementsInBaseModel ));
                }
            }
            //System.out.println("Total statements (assertions): " + cntStatements);
            //System.out.println("Statements in base model (assertions): " + cntStatements);
        }
    
    	return triples;
    }
    
    
    
    
    public static void handle(OntModel model, CCodeModelContainer writer) {
		
		TemplateString contents = new TemplateString(implementationTemplateName);		
		TemplateString ontTriple = new TemplateString(COntologyTriple);		
		TemplateString opEnumDecl = new TemplateString(CObjectPropertyEnumDeclaration); 
		contents.replaceTag("<@headerfile@>", writer.getHeaderFile());
		Map map = model.getBaseModel().getNsPrefixMap(); // MAP

		String namespace = (String) map.get("");
		
		ExtendedIterator itr = model.listNamedClasses();
		
				
		
		// this loop generates the create_ontology function		
		
		LinkedList<LinkedList<String>> triples = getTripleOntology(model);
	    for (LinkedList<String> list : triples) {
	    	String subject = list.get(0);
	    	String predicate = list.get(1);
	    	String object = list.get(2).replace("\n", "\\n");
	    	ontTriple.replaceAll2("<@subject@>", subject);
	    	ontTriple.replaceAll2("<@predicate@>", predicate);
	    	ontTriple.replaceAll2("<@object@>", object);	    	  	
	    	
	    	contents.insertIntoPlaceholder("<%ontologytriples%>", ontTriple.getString());
	    	ontTriple.resetToInitValue();
	   
	    }
	    contents.replaceAll("<@namespace@>", namespace);
	    contents.removeTags();
	    TemplateString headerHeaderContents = new TemplateString(
				headerHeaderTemplateName);
		
		headerHeaderContents.replaceAll("<@namespace@>", namespace);
		
		if (AppConfiguration.getTripleStore().toLowerCase().equals("disabled")){
			headerHeaderContents.replaceAll("<@triplestore@>", "");			
		} else {			
			headerHeaderContents.replaceAll("<@triplestore@>", "#define TRIPLESTORE = 1");
		}
		
		
		ExtendedIterator i = model.listObjectProperties();
		if (i.hasNext()){			
			while(i.hasNext()){
		
				ObjectProperty op = (ObjectProperty) i.next();
			
				if (i.hasNext()){
					opEnumDecl.insertIntoPlaceholder("<%objectpropertyname%>", op.getLocalName().toUpperCase() +",\n");
				} else {
					opEnumDecl.insertIntoPlaceholder("<%objectpropertyname%>", op.getLocalName().toUpperCase());
				}
			}
			opEnumDecl.removeTags();
			headerHeaderContents.insertIntoPlaceholder("<%objectpropertyenum%>", opEnumDecl.getString());
		}
			
		
		headerHeaderContents.replaceTag("<@headerfile@>", writer.getHeaderFile());
		headerHeaderContents.removeTags();
		writer.addDef(headerHeaderContents.getString());
		writer.addImpl(contents.getString());	
	
		
		i = model.listNamedClasses();

		while (i.hasNext()) {			
			OntClass oc = (OntClass) i.next();

			boolean hasThing = false;  
			
			ExtendedIterator iterator = oc.listSuperClasses();
			
			OntClass ontClass;
			while (iterator.hasNext()) {
							
				ontClass = (OntClass) iterator.next();		
					
				if (ontClass.getLocalName() != null) {			
					if (ontClass.getLocalName().equals("Thing")) {			
						hasThing = true;
						break;
					}
				}
				
			}
			
			if (!oc.isIndividual() && oc.getLocalName() != null && (!oc.getLocalName().equalsIgnoreCase("Nothing")) && (hasThing == true)) {				
				CClassHandler.handle(oc, writer);				
						
			}	
		
		
		}
		
		
	}

}
