/**
 * @file ss_subscribe.h
 * @Author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   05 DEcember, 2009
 * @brief  Interface for subscribe functions and subscribe container.
 *
 *
 * @section LICENSE
 *
 * SmartSlog KP Library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SmartSlog KP Library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SmartSlog KP Library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 *
 * @section DESCRIPTION
 *
 * ss_subscribe.h - interface for subscribe functions.
 * This file is part of SmartSlog KP Library.
 */


#include <stdio.h>
#include "utils/list.h"
#include "utils/kp_bool.h"
#include <kpilow/kpi_low.h>
#include "structures.h"


#ifndef _SS_SUBSCRIBE_H
#define	_SS_SUBSCRIBE_H


/******************************************************************************/
/****************************** Structures list *******************************/

/// @cond INTERNAL_STRUCTURES

/** @brief  Subscription data for subscription container @see subscription_container_s. */
typedef struct subscription_data_s {
    individual_t *ind;  /**< individual for subscribe. */
    list_t *properties; /**< individual properties for subscribe. */
} subscription_data_t;

/// @endcond


/// @cond EXTERNAL_STRUCTURES

/** @brief  Container contains data for subscribe. */
typedef struct subscription_container_s {
    int rtti;                   /**< Run-time type information. */
    list_t *sbrc_data;          /**< Subscription data list (@see subscription_data_s). */
    int subscription_id;        /**< Identifier. */
    int subscription_type;      /**< Type. */
    void (*callback)(struct subscription_container_s*, list_t*); /**< Callback function. */
    bool is_updated;            /**< Update flag. */
    ss_subs_info_t info;        /**< KPI Low information of subscription. */

} subscription_container_t;

/// @endcond


/******************************************************************************/
/****************************** Functions list ********************************/

/// @cond EXTERNAL_FUNCTIONS

subscription_container_t* new_subscription_container();
void free_subscription_container(subscription_container_t *container);


int add_individual_to_subscription_container(subscription_container_t *container,
        individual_t *ind, list_t *properties);
void set_subscription_container_callback(subscription_container_t *container,
        void (*func)(subscription_container_t *, list_t *));

int ss_subscribe_container(subscription_container_t *container, bool is_asynchronous);
int ss_unsubscribe_container(subscription_container_t *container);
int wait_subscribe(subscription_container_t *container);
void stop_all_subscriptions();

/// @endcond



/// @cond INTERNAL_FUNCTIONS

subscription_data_t* new_subscription_data(individual_t *ind, list_t *properties);
void free_subscription_data(subscription_data_t *data, bool free_prop_list);


/// @endcond


#endif	/* _SS_SUBSCRIBE_H */